using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Text.RegularExpressions;
using System.Data.SQLite;
//#Include:System.Data.dll
//#Include:System.Data.SQLite.dll

namespace PandoraBox
{
    public class CBTEXTPlugin : IPluginMacro
    {
        Regex _block = new Regex(@"\[(GOOGLE|BING|YANDEX)(FREE){0,1}TEXT-(\d+)-(\d+)(?<mp>#\d+)*\]", RegexOptions.Compiled);
        Regex _customBlock = new Regex(@"\[MYCB\(([^)]+)\)(FREE){0,1}TEXT-(\d+)-(\d+)(?<mp>#\d+)*\]", RegexOptions.Compiled);

        //сколько статей взять в работу / how many articles to take 
        int _count = 10;

        public string Execute(string template, PluginMacroArgs args)
        {
            template = Load(template, _block, args);
            template = Load(template, _customBlock, args);
            return template;
        }

        private string Load(string template, Regex regex, PluginMacroArgs args)
        {
            return regex.Replace(template, delegate (Match match)
            {
                var type = match.Groups[1].Value + ".Directory";
                String typePath = GetIniParameter(type, args);
                bool dbMode = GetIniParameter("Main.UseDatabase", args) == "1";
                String key = CleanKey(GetKey(args, match));
                bool isFreeText = match.Groups[2].Value.Length > 0;
                var lines = dbMode
                    ? GetDBLines(args, GetIniParameter("Main.DataFolder", args), typePath.Replace(".", "_"), key)
                    : GetFileText(Path.Combine(GetIniParameter("Main.DataFolder", args), typePath, GetSub(key.Trim()), key));
                if (lines == null) //текста по ключу нет, выводим стандартным макросом текста
                    return (isFreeText ? "[FREETEXT-" : "[TEXT-") + match.Groups[3].Value + "-" + match.Groups[4].Value + "]";
                lines = Shuffle(lines);
                var sentances = new List<String>();
                for (int i = 0; i < _count && i < lines.Length; i++)
                    sentances.AddRange(lines[i].Split('|'));
                sentances = Shuffle(sentances);
                int take = Rnd.Next(int.Parse(match.Groups[3].Value), int.Parse(match.Groups[4].Value));
                if (isFreeText)
                    return String.Join(" ", sentances.Take(take));
                return "{INSERTKEYS}" + String.Join(" ", sentances.Take(take)) + "{/INSERTKEYS}";
            });
        }

        private string GetKeysFile(PluginMacroArgs args)
        {
            return args.Context["$dbname"] as String ?? "content.db";
        }

        private string[] GetFileText(String file)
        {
            if (!File.Exists(file)) return null;
            var lines = File.ReadAllLines(file).Where(x => x.Length > 0).ToArray();
            if (lines.Length > 0)
                return lines;
            return null;
        }

        private string[] GetDBLines(PluginMacroArgs args, String dbDir, String table, String key)
        {
            //connect
            var contextKey = "SQLiteConnect";
            SQLiteConnection connection = args.Context[contextKey] as SQLiteConnection;
            if (connection == null)
            {
                connection = new SQLiteConnection("Data Source = \"" + dbDir + "/" + GetKeysFile(args) + "\"; Journal Mode=Off;");
                connection.Open();
                args.Context[contextKey] = connection;
            }

            //reader command
            contextKey = "SQLiteCommand" + table;
            SQLiteCommand reader = args.Context[contextKey] as SQLiteCommand;
            if (reader == null)
            {
                reader = new SQLiteCommand("select content from '" + table + "' where key = :key", connection);
                reader.Parameters.Add(new SQLiteParameter("key", ""));
                args.Context[contextKey] = reader;
                if (!TableExists(table, connection)) reader.CommandText = null;
            }
            if (reader.CommandText == null) return new string[0]; //no table

            reader.Parameters[0].Value = key;
            var content = (reader.ExecuteScalar() as string) ?? "";
            var lines = content.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries);
            if (lines.Length > 0)
                return lines;
            return null;
        }

        private bool TableExists(String table, SQLiteConnection connection)
        {
            return new SQLiteCommand("SELECT count(*) FROM sqlite_master WHERE name='" + table + "'", connection).ExecuteScalar().ToString() == "1";
        }

        private string GetIniParameter(String name, PluginMacroArgs args)
        {
            String root = args.Context[name] as String;
            if (root == null)
            {
                root = File.ReadAllLines("ContentBox.ini")
                    .FirstOrDefault(x => x.StartsWith(name + "=", StringComparison.InvariantCultureIgnoreCase));
                root = root != null ? root.Split('=')[1] : "";
                args.Context[name] = root;
            }
            return root;
        }

        private string GetSub(string key)
        {
            return key.Length <= 2 ? key : key.Substring(0, 2).Trim();
        }

        private string GetKey(PluginMacroArgs args, Match match)
        {
            int keyIndex;
            string pos = null;
            var result = args.Key;
            if (args.KeyParts != null && (pos = match.Groups["mp"].Value) != "" && (keyIndex = int.Parse(pos.Trim('#'))) > 0)
                result = args.KeyParts[keyIndex > args.KeyParts.Length ? 0 : keyIndex - 1];
            return result;
        }

        private string CleanKey(String key)
        {
            String key1 = key;
            key1 = String.Join("", key1.Split(Path.GetInvalidPathChars()));
            key1 = String.Join("", key1.Split(Path.GetInvalidFileNameChars()));
            return key1;
        }

        private static List<T> Shuffle<T>(List<T> items)
        {
            if (items.Count < 2) return items;

            for (int i = items.Count - 1; i > 0; i--)
            {
                int j = Rnd.InnerObject.Next(i); //Фишер–Йетс
                T temp = items[i];
                items[i] = items[j];
                items[j] = temp;
            }
            return items;
        }

        private static T[] Shuffle<T>(T[] items)
        {
            if (items.Length < 2) return items;

            for (int i = items.Length - 1; i > 0; i--)
            {
                int j = Rnd.InnerObject.Next(i); //Фишер–Йетс
                T temp = items[i];
                items[i] = items[j];
                items[j] = temp;
            }
            return items;
        }

        public ushort Level { get { return 5; } }
        public String Name { get { return "CBTEXTPlugin"; } }
    }
}